import Constants, Globals, exceptions, time

class Piece:
    """A simple class representing a piece"""
    def __init__(self, blocks, color = 0x00FF00):
        self.blocks = blocks
        self.color = color
        self.hover_time = time.time()
        self.state = 1
        self.num_states = len(blocks) - 1

    def rotate(self, direction):
        old_state = self.state
        #FIXME: I'm startin' to arbitrarily grab stuff from everywhere
        self.erase(Globals.play_field)
        if direction == Constants.RIGHT:
            self.state -= 1
            if self.state < 1:
                self.state += self.num_states
        else:
            self.state += 1
            if self.state > self.num_states:
                self.state = 1
        try:
            self.collision_detect()
        except Collision:
            self.state = old_state


    def move(self, value, direction, collision_detect=True):
        # move pivot
        pivot = self.blocks[0]
        pivot[direction[1]] += value * direction[0]

        # move rest of blocks
        for blocks in self.blocks[1:]:
            for block in blocks:
                block[direction[1]] += value * direction[0]
        try: self.collision_detect()
        except Collision, e:
            if collision_detect:
                # We had a collision, undo what we did
                pivot[direction[1]] -= value * direction[0]
                for blocks in self.blocks[1:]:
                    for block in blocks:
                        block[direction[1]] -= value * direction[0]

                # was the collision while moving down?
                if direction == Constants.DOWN:
                    raise Collision, "Done!"
                else:
                    raise Collision
            else:
                return

    def collision_detect(self):
        """See if we've bumped into anything in the playfield"""
        for x, y in ([self.blocks[0]] + self.blocks[self.state]):
            try:
                if x < 0 or x > (len(Globals.play_field) - 1):
                    raise Collision
                if y < 0:
                    # if block off of playfield, skip it
                    continue
                #FIXME: set colors in Constants
                if Globals.play_field[x][y][0] != 0x000000:
                    # We've collided on the playfield
                    raise Collision
            except IndexError:
                # hit boundary of play field
                raise Collision

    def drop(self):
        """Move piece downward if enough time has passed"""
        # check time passed in milliseconds and compare to
        # Globals.current_speed.  If passed, move piece down.
        time_passed = int((time.time() - self.hover_time) * 1000)
        if time_passed > Globals.current_speed:
            self.move(1, Constants.DOWN)
            self.hover_time = time.time()

    def draw(self, play_box, color=None):
        """Draw piece onto the screen, pygame.display.flip() must be called"""
        # get color if passed, otherwise default to piece color, get
        # play field surface from the playfield structure, draw each
        # block.
        if not color: color = self.color
        # draw pivot and current state blocks
        for block in ([self.blocks[0]] + self.blocks[self.state]):
            try:
                # if block off top of play field, don't draw it.
                if block[1] < 0 or block[0] < 0:
                    continue
                play_field_square = play_box[block[0]][block[1]]
                play_field_square[1].fill(color)
                play_field_square[0] = color
            except IndexError:
                print "Index crash!  X: %d, Y: %d" % (block[0],block[1])

    def erase(self, play_box):
        """Erase piece from current position"""
        # erase pivot and blocks
        for block in ([self.blocks[0]] + self.blocks[self.state]):
            try:
                if block[1] < 0 or block[0] < 0 or \
                   block[0] > (len(Globals.play_field) - 1):
                    continue
                play_field_square = play_box[block[0]][block[1]]
                play_field_square[1].fill(0x000000)
                play_field_square[0] = 0x000000
            except IndexError:
                print "Value of block[0]: %d" % block[0]
                print "Value of block[1]: %d" % block[1]
                raise IndexError
    def reset_hover_time(self):
        """Sets the hover time of the current piece to the current time"""
        self.hover_time = time.time()


class Collision(exceptions.Exception):
    def __init__(self, error=None):
        exceptions.Exception.__init__(self)
        self.error = error


class Player:
    """Keeps track of player statistics"""
    def __init__(self):
        self.score = 0
        self.lines = 0
        self.games = 0

    def add_score():
        pass

    def clear_score():
        pass

    def add_lines():
        pass

    def clear_lines():
        pass
